# ../addons/eventscripts/looneybotping/looneybotping.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# Python Imports
from random import randint


# Eventscripts Imports
import es
import cfglib
import gamethread

# =============================================================================
# >> ADDON REGISTRATION/INFORMATION
# =============================================================================
info = es.AddonInfo()
info.name = 'LooneyBotPing'
info.basename = info.name.lower()
info.version = '1.2'
info.url = 'http://addons.eventscripts.com/addons/view/LooneyBotPing'
info.author = 'Satoon101'
info.variable = info.basename + '_version'
es.ServerVar(info.variable, info.version,
        info.name + ' Version').set(info.version)
es.ServerVar(info.variable).makepublic()


# =============================================================================
# >> GLOBAL VARIABLES
# =============================================================================
config = cfglib.AddonCFG(es.getAddonPath(info.basename) + '/lbp_config.cfg')


# =============================================================================
# >> CLASSES
# =============================================================================
class BotPingHandler:
    class BotIndexes:
        def __init__(self, userid):
            self.userid = userid
            self.index = es.getindexfromhandle(es.getplayerhandle(self.userid))

        def pingLoop(self):
            if not es.getuserid(self.userid):
                return

            es.setindexprop(bot.player_manager,
                bot.prop_base + '.%.3i' % self.index,
                randint(int(bot.min_ping), int(bot.max_ping)))

            gamethread.delayedname(randint(int(bot.min_delay),
             int(bot.max_delay)), 'lbp_Bot_%s_Ping' % self.userid,
             self.pingLoop)

        def removeLoop(self):
            gamethread.cancelDelayed('lbp_Bot_%s_Ping' % self.userid)

    def __init__(self):
        self.BotList = {}
        config.text(self.cfgText('LooneyBotPing Options'))
        self.max_ping = config.cvar('lbp_max_ping', 55,
                'The highest ping that will be shown for Bots')
        self.min_ping = config.cvar('lbp_min_ping', 25,
                'The lowest ping that will be shown for Bots')
        self.max_delay = config.cvar('lbp_max_delay', 3,
                'Longest delay between ping updates for Bots')
        self.min_delay = config.cvar('lbp_min_delay', 2,
                'Shortest delay between ping updates for Bots')
        config.write()
        gamethread.queue(self.doLoad)

    def doLoad(self):
        self.player_manager, self.prop_base = self.getPlayerManager()
        if self.player_manager is None:
            raise NotImplementedError, ('LooneyBotPing does not support game' +
                ' "' + es.getGameName() + '"'), es.unload('looneybotping')
        for userid in self.getAllBotUserids():
            self.getBot(userid).pingLoop()

    def getBot(self, userid):
        userid = str(userid)
        if not userid in self.BotList:
            self.BotList[userid] = self.BotIndexes(userid)
        return self.BotList[userid]

    def playerActivate(self, userid):
        if es.isbot(userid):
            self.getBot(userid).pingLoop()

    def playerDisconnect(self, userid):
        if userid in self.BotList:
            self.getBot(userid).removeLoop()
            del self.BotList[userid]

    def clearList(self):
        for userid in self.BotList:
            self.getBot(userid).removeLoop()
        self.BotList.clear()

    @staticmethod
    def cfgText(text):
        stars = '*' * (len(text) + 6)
        return stars + '\n// ** ' + text + ' **\n// ' + stars

    @staticmethod
    def getAllBotUserids():
        for userid in es.getUseridList():
            if es.isbot(userid):
                yield userid

    @staticmethod
    def getPlayerManager():
        entlist = es.createentitylist()
        for index in entlist:
            for prop in entlist[index]:
                if 'm_iping' in prop.lower():
                    return (index, prop.rsplit('.', 1)[0])
        return None, None

bot = BotPingHandler()


# =============================================================================
# >> LOAD & UNLOAD
# =============================================================================
def load():
    es.mexec('..' + config.cfgpath.replace(config.gamedir, '', 1))


def unload():
    bot.clearList()


# =============================================================================
# >> GAME EVENTS
# =============================================================================
def es_map_start(ev):
    bot.clearList()


def player_disconnect(ev):
    bot.playerDisconnect(ev['userid'])


def player_activate(ev):
    bot.playerActivate(ev['userid'])
